/****** Object:  UserDefinedFunction [dbo].[MinDate]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: MinDate
--	Create date: 12/02/2020
--
--	DESCRIPTION
--	Returns the minimum date among those passed as a parameter to the function
--
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid credentials
-- --------------------------------------------
CREATE FUNCTION [dbo].[MinDate]
(
	@Date1 DATETIME,
	@Date2 DATETIME,
	@Date3 DATETIME,
	@Date4 DATETIME,
	@Date5 DATETIME
)
RETURNS DATETIME
AS
BEGIN
    DECLARE @RESULT DATETIME

	SELECT @RESULT = MIN(DT) 
        FROM
        (
            VALUES (@Date1),
                   (@Date2),
                   (@Date3),
                   (@Date4),
                   (@Date5)
        ) AS T(DT)

    RETURN @RESULT
END
GO
/****** Object:  Table [dbo].[Contact]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[Contact](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[User1ID] [int] NOT NULL,
	[User2ID] [int] NOT NULL,
	[User1State] [varchar](10) NOT NULL,
	[User2State] [varchar](10) NOT NULL,
	[CreationDate] [datetime] NOT NULL,
	[User1LastUpdate] [datetime] NOT NULL,
	[User2LastUpdate] [datetime] NOT NULL,
 CONSTRAINT [PK_Contact] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, OPTIMIZE_FOR_SEQUENTIAL_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO
/****** Object:  Table [dbo].[ContactRequest]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[ContactRequest](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[RequestorID] [int] NOT NULL,
	[RecipientID] [int] NOT NULL,
	[RequestorState] [varchar](10) NOT NULL,
	[RecipientState] [varchar](10) NOT NULL,
	[CreationDate] [datetime] NOT NULL,
	[RequestorLastUpdate] [datetime] NOT NULL,
	[RecipientLastUpdate] [datetime] NOT NULL,
 CONSTRAINT [PK_ContactRequest] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, OPTIMIZE_FOR_SEQUENTIAL_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO
/****** Object:  Table [dbo].[Message]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[Message](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[OwnerID] [int] NOT NULL,
	[SenderID] [int] NOT NULL,
	[RecipientID] [int] NOT NULL,
	[CreationDate] [datetime] NOT NULL,
	[ModifyDate] [datetime] NOT NULL,
	[Message] [nvarchar](250) NOT NULL,
	[State] [varchar](10) NOT NULL,
 CONSTRAINT [PK_Message] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, OPTIMIZE_FOR_SEQUENTIAL_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO
/****** Object:  Table [dbo].[User]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[User](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[EMail] [nvarchar](260) NOT NULL,
	[Nickname] [nvarchar](100) NOT NULL,
	[Password] [varbinary](max) NOT NULL,
	[CreationDate] [datetime] NOT NULL,
	[LastUpdate] [datetime] NOT NULL,
 CONSTRAINT [PK_User] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, OPTIMIZE_FOR_SEQUENTIAL_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
ALTER TABLE [dbo].[Contact] ADD  DEFAULT (getdate()) FOR [CreationDate]
GO
ALTER TABLE [dbo].[Contact] ADD  DEFAULT (getdate()) FOR [User1LastUpdate]
GO
ALTER TABLE [dbo].[Contact] ADD  DEFAULT (getdate()) FOR [User2LastUpdate]
GO
ALTER TABLE [dbo].[ContactRequest] ADD  DEFAULT (getdate()) FOR [CreationDate]
GO
ALTER TABLE [dbo].[ContactRequest] ADD  DEFAULT (getdate()) FOR [RequestorLastUpdate]
GO
ALTER TABLE [dbo].[ContactRequest] ADD  DEFAULT (getdate()) FOR [RecipientLastUpdate]
GO
ALTER TABLE [dbo].[Message] ADD  DEFAULT (getdate()) FOR [CreationDate]
GO
ALTER TABLE [dbo].[Message] ADD  DEFAULT (getdate()) FOR [ModifyDate]
GO
ALTER TABLE [dbo].[User] ADD  DEFAULT (getdate()) FOR [CreationDate]
GO
ALTER TABLE [dbo].[User] ADD  DEFAULT (getdate()) FOR [LastUpdate]
GO
ALTER TABLE [dbo].[Contact]  WITH CHECK ADD  CONSTRAINT [FK_Contact_User1ID] FOREIGN KEY([User1ID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[Contact] CHECK CONSTRAINT [FK_Contact_User1ID]
GO
ALTER TABLE [dbo].[Contact]  WITH CHECK ADD  CONSTRAINT [FK_Contact_User2ID] FOREIGN KEY([User2ID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[Contact] CHECK CONSTRAINT [FK_Contact_User2ID]
GO
ALTER TABLE [dbo].[ContactRequest]  WITH CHECK ADD  CONSTRAINT [FK_ContactRequest_RecipientID] FOREIGN KEY([RecipientID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[ContactRequest] CHECK CONSTRAINT [FK_ContactRequest_RecipientID]
GO
ALTER TABLE [dbo].[ContactRequest]  WITH CHECK ADD  CONSTRAINT [FK_ContactRequest_RequestorID] FOREIGN KEY([RequestorID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[ContactRequest] CHECK CONSTRAINT [FK_ContactRequest_RequestorID]
GO
ALTER TABLE [dbo].[Message]  WITH CHECK ADD  CONSTRAINT [FK_Message_OwnerUserID] FOREIGN KEY([OwnerID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[Message] CHECK CONSTRAINT [FK_Message_OwnerUserID]
GO
ALTER TABLE [dbo].[Message]  WITH CHECK ADD  CONSTRAINT [FK_Message_RecipientUserID] FOREIGN KEY([RecipientID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[Message] CHECK CONSTRAINT [FK_Message_RecipientUserID]
GO
ALTER TABLE [dbo].[Message]  WITH CHECK ADD  CONSTRAINT [FK_Message_SenderUserID] FOREIGN KEY([SenderID])
REFERENCES [dbo].[User] ([ID])
GO
ALTER TABLE [dbo].[Message] CHECK CONSTRAINT [FK_Message_SenderUserID]
GO
ALTER TABLE [dbo].[Contact]  WITH CHECK ADD CHECK  (([User1State]='DELETED' OR [User1State]='CONFIRMED'))
GO
ALTER TABLE [dbo].[Contact]  WITH CHECK ADD CHECK  (([User2State]='DELETED' OR [User2State]='CONFIRMED'))
GO
ALTER TABLE [dbo].[ContactRequest]  WITH CHECK ADD CHECK  (([RecipientState]='REJECTED' OR [RecipientState]='CONFIRMED' OR [RecipientState]='PENDING'))
GO
ALTER TABLE [dbo].[ContactRequest]  WITH CHECK ADD CHECK  (([RequestorState]='CONFIRMED'))
GO
ALTER TABLE [dbo].[Message]  WITH CHECK ADD CHECK  (([OwnerID]=[SenderID] OR [OwnerID]=[RecipientID]))
GO
ALTER TABLE [dbo].[Message]  WITH CHECK ADD CHECK  (([OwnerID]=[SenderID] AND [State]='SENT' OR [OwnerID]=[RecipientID] AND ([State]='DELIVERED' OR [State]='PENDING')))
GO
/****** Object:  StoredProcedure [dbo].[Contact_Remove]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: Contact_Remove
--	Create date: 01/04/2020
--
--	DESCRIPTION
--	Remove a contact from a user's contact list
--
--	RETURNS
--	-1:		One or more parameter is missing
-- --------------------------------------------
CREATE PROCEDURE [dbo].[Contact_Remove]
	@UserID			INT,
	@UserContactID	INT
AS
BEGIN
	IF @UserID IS NULL
		RETURN -1

	-- Only one of the update should update a record
	UPDATE [dbo].[Contact]
	SET User1State = 'DELETED'
	WHERE [User1ID] = @UserID
			AND [User2ID] = @UserContactID

	UPDATE [dbo].[Contact]
	SET User2State = 'DELETED'
	WHERE [User2ID] = @UserID
			AND [User1ID] = @UserContactID
END
GO
/****** Object:  StoredProcedure [dbo].[Contact_SendRequest]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: ContactRequest_Send
--	Create date: 04/02/2020
--
--  DESCRIPTION
--	Insert a new contact request
--	
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid RequestorID
--	-3:		Invalid RecipientID
--	-4:		Contact request already exists
--	-5:		The recipient is already a contact for the requestor
-- --------------------------------------------
CREATE PROCEDURE [dbo].[Contact_SendRequest]
	@ID				INT OUTPUT,
	@RequestorID	INT,
	@RecipientID	INT
AS
BEGIN
	IF @RequestorID IS NULL OR @RecipientID IS NULL
		RETURN -1

	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @RequestorID)
		RETURN -2

	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @RecipientID)
		RETURN -3

	IF EXISTS(SELECT * FROM [dbo].[ContactRequest] WHERE RequestorID = @RequestorID AND RecipientID = @RecipientID)
		RETURN -4

	IF EXISTS(SELECT * FROM [dbo].[Contact] WHERE (User1ID = @RequestorID AND User2ID =  @RecipientID) 
													OR (User1ID = @RecipientID AND User2ID = @RequestorID))
		RETURN -5

	INSERT INTO [dbo].[ContactRequest](RequestorID, RecipientID, RequestorState, RecipientState)
	VALUES(@RequestorID, @RecipientID, 'CONFIRMED', 'PENDING')

	SET @ID = SCOPE_IDENTITY()
END
GO
/****** Object:  StoredProcedure [dbo].[ContactRequest_Confirm]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: ContactRequest_Confirm
--	Create date: 22/03/2020
--
--  DESCRIPTION
--	Confirm a contact request
--	
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		RequestID / RecipientID mismatch
-- --------------------------------------------
CREATE PROCEDURE [dbo].[ContactRequest_Confirm]
	@UserID			INT,
	@RequestID		INT
AS
BEGIN
	IF @UserID IS NULL OR @RequestID IS NULL
		RETURN -1
	
	IF NOT EXISTS(SELECT * FROM [dbo].[ContactRequest] WHERE RecipientID = @UserID AND ID = @RequestID)
		RETURN -2

	DECLARE @RequestorID	NUMERIC(10)
	SELECT @RequestorID = RequestorID FROM [dbo].[ContactRequest] WHERE ID = @RequestID

	DECLARE @MyTranCount INT = @@TRANCOUNT

	IF @MyTranCount = 0
		BEGIN TRAN

	BEGIN TRY
		UPDATE [dbo].[ContactRequest]
		SET RecipientState = 'CONFIRMED',
			RecipientLastUpdate = GETDATE()
		WHERE ID = @RequestID

		DELETE [dbo].[Contact]
		WHERE [User1ID] = @RequestorID AND [User2ID] = @UserID
				OR [User1ID] = @UserID AND [User2ID] = @RequestorID

		INSERT INTO [dbo].[Contact]([User1ID], [User2ID], [User1State], [User2State]) VALUES(@RequestorID, @UserID, 'CONFIRMED', 'CONFIRMED')

		IF @MyTranCount = 0
			COMMIT TRAN
	END TRY
	BEGIN CATCH
		IF @MyTranCount = 0
			ROLLBACK TRAN

		THROW
	END CATCH
END
GO
/****** Object:  StoredProcedure [dbo].[ContactRequest_Detail]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: ContactRequest_Detail
--	Create date: 22/03/2020
--
--  DESCRIPTION
--	Return the detail of the given contact request
--	
--	RETURNS
--	-1:		One or more parameter is missing
-- --------------------------------------------
CREATE PROCEDURE [dbo].[ContactRequest_Detail]
	@ID				INT
AS
BEGIN
	IF @ID IS NULL 
		RETURN -1

	SELECT CR.ID, CR.CreationDate, CR.RecipientID, CR.RecipientLastUpdate, CR.RecipientState,
			CR.RequestorID, CR.RequestorLastUpdate, CR.RequestorState
	FROM [dbo].[ContactRequest] CR
	WHERE CR.ID = @ID
END
GO
/****** Object:  StoredProcedure [dbo].[ContactRequest_Reject]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: ContactRequest_Reject
--	Create date: 22/03/2020
--
--  DESCRIPTION
--	Reject a contact request
--	
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		RequestID / RecipientID mismatch
-- --------------------------------------------
CREATE PROCEDURE [dbo].[ContactRequest_Reject]
	@UserID			INT,
	@RequestID		INT
AS
BEGIN
	IF @UserID IS NULL OR @RequestID IS NULL
		RETURN -1
	
	IF NOT EXISTS(SELECT * FROM [dbo].[ContactRequest] WHERE RecipientID = @UserID AND ID = @RequestID)
		RETURN -2

	UPDATE [dbo].[ContactRequest]
	SET RecipientState = 'REJECTED',
		RecipientLastUpdate = GETDATE()
	WHERE ID = @RequestID

END
GO
/****** Object:  StoredProcedure [dbo].[ContactRequest_Send]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: ContactRequest_Send
--	Create date: 04/02/2020
--
--  DESCRIPTION
--	Insert a new contact request
--	
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid RequestorID
--	-3:		Invalid RecipientID
--	-4:		Contact request already exists
--	-5:		The recipient is already a contact for the requestor
-- --------------------------------------------
CREATE PROCEDURE [dbo].[ContactRequest_Send]
	@ID				INT OUTPUT,
	@RequestorID	INT,
	@RecipientID	INT
AS
BEGIN
	IF @RequestorID IS NULL OR @RecipientID IS NULL
		RETURN -1

	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @RequestorID)
		RETURN -2

	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @RecipientID)
		RETURN -3

	IF EXISTS(SELECT * FROM [dbo].[ContactRequest] WHERE RequestorID = @RequestorID AND RecipientID = @RecipientID AND RecipientState = 'PENDING' )
		RETURN -4

	IF EXISTS(SELECT * FROM [dbo].[Contact] WHERE ((User1ID = @RequestorID AND User2ID =  @RecipientID) 
													OR (User1ID = @RecipientID AND User2ID = @RequestorID))
													AND User1State = 'CONFIRMED' AND User2State = 'CONFIRMED')
		RETURN -5

	INSERT INTO [dbo].[ContactRequest](RequestorID, RecipientID, RequestorState, RecipientState)
	VALUES(@RequestorID, @RecipientID, 'CONFIRMED', 'PENDING')

	SET @ID = SCOPE_IDENTITY()
END
GO
/****** Object:  StoredProcedure [dbo].[Message_Send]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: Message_Send
--	Create date: 03/02/2020
--
--  DESCRIPTION
--	Insert a new message
--	
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid SenderID 
--	-3:		Invalid RecipientID
-- --------------------------------------------
CREATE PROCEDURE [dbo].[Message_Send]
	@ID				INT 		OUTPUT,
	@SenderID		INT,
	@RecipientID	INT,
	@Message		NVARCHAR(250)
AS
BEGIN
	IF ISNULL(@Message, '') = '' OR @SenderID IS NULL OR @RecipientID IS NULL
		RETURN -1
	
	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @SenderID)
		RETURN -2

	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @RecipientID)
		RETURN -3

	DECLARE @MY_TRANCOUNT INT = @@TRANCOUNT

	IF @MY_TRANCOUNT = 0
		BEGIN TRAN

	BEGIN TRY
		INSERT INTO [dbo].[Message](OwnerID, SenderID, RecipientID, [Message], [State])
		VALUES(@SenderID, @SenderID, @RecipientID, @Message, 'SENT')

		SET @ID = SCOPE_IDENTITY()

		INSERT INTO [dbo].[Message](OwnerID, SenderID, RecipientID, [Message], [State])
		VALUES(@RecipientID, @SenderID, @RecipientID, @Message, 'PENDING')

		IF @MY_TRANCOUNT = 0
			COMMIT TRAN
	END TRY
	BEGIN CATCH
		IF @MY_TRANCOUNT = 0
			ROLLBACK TRAN

		THROW
	END CATCH
END
GO
/****** Object:  StoredProcedure [dbo].[Message_SetDeleted]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: Message_SetDeleted
--	Create date: 03/02/2020
--
--	DESCRIPTION
--	Set a message as deleted. Depending on the user
--	who makes the action, set the status appropriately.
--
--	RETURNS
--	-1:		One ore more parameter is missing
--	-2:		Invalid UserID
-- --------------------------------------------
CREATE PROCEDURE [dbo].[Message_SetDeleted]
	@ID			INT,
	@UserID		INT
AS
BEGIN
	IF @ID IS NULL OR @UserID IS NULL
		RETURN -1

	--DECLARE @RecipientID	INT
	--DECLARE @SenderID		INT
	
	--SELECT	@RecipientID	= RecipientID,
	--		@SenderID		= SenderID
	--FROM [dbo].[Message]
	--WHERE ID = @ID

	--IF @UserID <> @SenderID AND @UserID <> @RecipientID
	--	RETURN -2

	--IF @UserID = @SenderID
	--BEGIN
	--	-- If the sender deletes a message, it will be also deleted for the recipient
	--	UPDATE [dbo].[Message] SET SenderState = 'DELETED', RecipientState = 'DELETED' WHERE ID = @ID
	--END
	--ELSE
	--BEGIN
	--	-- The recipient deletes the message for himself only
	--	UPDATE [dbo].[Message] SET RecipientState = 'DELETED' WHERE ID = @ID
	--END
END
GO
/****** Object:  StoredProcedure [dbo].[Message_SetDelivered]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: Message_SetDelivered
--	Create date: 03/02/2020
--
--	DELETE
--	Set a message as delivered
--	
--	RETURNS
--	-1:		One ore more parameter is missing
--	-2:		Invalid message state or non existent message
-- --------------------------------------------
CREATE PROCEDURE [dbo].[Message_SetDelivered]
	@ID			INT
AS
BEGIN
	IF @ID IS NULL
		RETURN -1

	--IF NOT EXISTS(SELECT * FROM [dbo].[Message] WHERE ID = @ID AND SenderState = 'SENT' AND RecipientState = 'PENDING')
	--	RETURN -2

	--UPDATE [dbo].[Message] 
	--SET RecipientState = 'DELIVERED'
	--WHERE ID = @ID
END
GO
/****** Object:  StoredProcedure [dbo].[User_GetContactRequests]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: User_GetContactRequests
--	Create date: 09/02/2020
--
--	DESCRIPTION
--	Get the contact requests list of the given user
--
--	RETURNS
--	-1:		One or more parameter is missing
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_GetContactRequests]
	@UserID		INT
AS
BEGIN
	IF @UserID IS NULL
		RETURN -1

	SELECT CR.ID, CR.RequestorID AS UserID, U.EMail, U.Nickname
	FROM [dbo].ContactRequest CR
	INNER JOIN [dbo].[User] U ON CR.RequestorID = U.ID
	WHERE CR.RecipientID = @UserID
			AND CR.RecipientState = 'PENDING'
	ORDER BY U.Nickname
END
GO
/****** Object:  StoredProcedure [dbo].[User_GetContacts]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: User_GetContacts
--	Create date: 09/02/2020
--
--	DESCRIPTION
--	Get the contacts list of the given user
--
--	RETURNS
--	-1:		One or more parameter is missing
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_GetContacts]
	@UserID			INT
AS
BEGIN
	IF @UserID IS NULL
		RETURN -1

	SELECT T.ID, T.UserID, T.EMail, T.Nickname, ISNULL(PENDING_MSG.MessageCount, 0) AS PendingMessages
	FROM
	(
		SELECT C.ID, C.User2ID UserID, U.EMail, U.Nickname
		FROM [dbo].Contact C
		INNER JOIN [dbo].[User] U ON C.User2ID = U.ID
		WHERE C.User1ID = @UserID 
				AND C.User1State = 'CONFIRMED'
				AND C.User2State = 'CONFIRMED'
		UNION
		SELECT C.ID, C.User1ID AS UserID, U.EMail, U.Nickname
		FROM [dbo].Contact C
		INNER JOIN [dbo].[User] U ON C.User1ID = U.ID
		WHERE C.User2ID = @UserID 
				AND C.User1State = 'CONFIRMED'
				AND C.User2State = 'CONFIRMED'
	) AS T
	LEFT JOIN (SELECT M.SenderID, COUNT(*) AS MessageCount 
				FROM [dbo].[Message] M 
				WHERE M.RecipientID = @UserID
					AND M.[State] = 'PENDING'
				GROUP BY M.SenderID
	) PENDING_MSG ON T.UserID = PENDING_MSG.SenderID
	ORDER BY Nickname
END
GO
/****** Object:  StoredProcedure [dbo].[User_GetMessages]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: User_GetMessages
--	Create date: 13/02/2020
--
--	DESCRIPTION
--	Get the contacts list of messages of a given user
--
--	RETURNS
--	-1:		One or more parameter is missing
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_GetMessages]
	@UserID				INT,
	@RecipientID		INT
AS
BEGIN
	IF @UserID IS NULL
		RETURN -1

	SELECT M.ID, M.CreationDate, M.ModifyDate, M.[Message],
			M.SenderID, SND.EMail AS SenderEMail, SND.Nickname AS SenderNickName,
			M.RecipientID, REC.EMail AS RecipientEMail, REC.Nickname AS RecipientNickname, 
			M.[State]
	FROM [dbo].[Message] M
	INNER JOIN [dbo].[User] SND ON M.SenderID = SND.ID
	INNER JOIN [dbo].[User] REC ON M.RecipientID = REC.ID
	WHERE M.OwnerID = @UserID
			AND ((M.SenderID = @UserID AND M.RecipientID = @RecipientID)
					OR (M.SenderID = @RecipientID AND M.RecipientID = @UserID))
	ORDER BY M.CreationDate


	UPDATE [dbo].[Message] 
	SET [State] = 'DELIVERED' 
	WHERE SenderID = @RecipientID 
		AND RecipientID = @UserID
		AND [State] = 'PENDING'
END
GO
/****** Object:  StoredProcedure [dbo].[User_InsUpd]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: Contact_InsUpd
--	Create date: 03/02/2020
--
--	DESCRIPTION
--	Insert / Update a user in the database
--
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		EMail is already in use
--	-3:		Nickname is already in use
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_InsUpd]
	@ID			INT				OUTPUT,
	@EMail		NVARCHAR(260),
	@Nickname	NVARCHAR(100),
	@Password	NVARCHAR(100)
AS
BEGIN
	IF ISNULL(@EMail, '') = '' OR ISNULL(@NickName, '') = '' OR @Password IS NULL
		RETURN -1

	IF EXISTS(SELECT * FROM [dbo].[User] WHERE EMail = @EMail)
		RETURN -2

	IF EXISTS(SELECT * FROM [dbo].[User] WHERE Nickname = @Nickname)
		RETURN -3

	DECLARE @PasswordHash VARBINARY(MAX)
	SET @PasswordHash = HASHBYTES('SHA2_512', @Password)

	IF ISNULL(@ID, 0) = 0
	BEGIN
		INSERT INTO [dbo].[User](EMail, Nickname, [Password]) VALUES(@EMail, @Nickname, @PasswordHash)
		SET @ID = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [dbo].[User] SET EMail		= @EMail, 
								Nickname	= @Nickname, 
								[Password]	= @PasswordHash
		WHERE ID = @ID
	END

	SELECT @ID AS ID
END
GO
/****** Object:  StoredProcedure [dbo].[User_Login]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: User_Login
--	Create date: 08/02/2020
--
--	DESCRIPTION
--	User login
--
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid credentials
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_Login]
	@EMail		NVARCHAR(260),
	@Password	NVARCHAR(100)
AS
BEGIN
	IF ISNULL(@EMail, '') = '' OR @Password IS NULL
		RETURN -1

	DECLARE @PasswordHash VARBINARY(MAX)
	SET @PasswordHash = HASHBYTES('SHA2_512', @Password)

	SELECT ID, EMail, Nickname 
	FROM [dbo].[User] 
	WHERE EMail = @EMail AND [Password] = @PasswordHash

	IF @@ROWCOUNT = 0
		RETURN -2
END
GO
/****** Object:  StoredProcedure [dbo].[User_Search]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: User_Search
--	Create date: 15/03/2020
--
--	DESCRIPTION
--	Show the users list, excluding those already in the contact list
--	of the caller
--
--	RETURNS
--	-1:		One or more parameter is missing
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_Search]
	@UserID				INT,
	@SearchText NVARCHAR(260)
AS
BEGIN
	IF @UserID IS NULL
		RETURN -1

	SELECT U.ID, U.Nickname, U.EMail
	FROM [dbo].[User] U
	WHERE U.ID <> @UserID
		AND ((U.EMail LIKE '%' + @SearchText + '%')
			OR (U.Nickname LIKE '%' + @SearchText + '%')
			OR @SearchText IS NULL)
		AND NOT EXISTS(SELECT * FROM [dbo].Contact C1 WHERE C1.User1ID = @UserID AND C1.User2ID = U.ID AND C1.User1State = 'CONFIRMED' AND C1.User2State = 'CONFIRMED')
		AND NOT EXISTS(SELECT * FROM [dbo].Contact C2 WHERE C2.User1ID = U.ID AND C2.User2ID = @UserID AND C2.User1State = 'CONFIRMED' AND C2.User2State = 'CONFIRMED')
	ORDER BY U.Nickname

END
GO
/****** Object:  StoredProcedure [dbo].[User_Synchronize]    Script Date: 11-Apr-20 5:48:05 PM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- --------------------------------------------
--	SP: User_Synchronize
--	Create date: 12/02/2020
--
--	DESCRIPTION
--	User synchronization (Contacts, ContactRequests, 
--	Messages)
--
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid credentials
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_Synchronize]
	@ClientID		INT,
	@FromDate		DATETIME
AS
BEGIN
	/*SELECT C.ID
	FROM Contact C
	*/
		--	C.LastUpdate >= @FromDate
	RETURN
END
GO
