﻿/*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
*****************************************************************************/

-- --------------------------------------------
--	SP: Message_Send
--	Create date: 03/02/2020
--
--  DESCRIPTION
--	Insert a new message
--	
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		Invalid SenderID 
--	-3:		Invalid RecipientID
-- --------------------------------------------
CREATE PROCEDURE [dbo].[Message_Send]
	@ID				INT 		OUTPUT,
	@SenderID		INT,
	@RecipientID	INT,
	@Message		NVARCHAR(250)
AS
BEGIN
	IF ISNULL(@Message, '') = '' OR @SenderID IS NULL OR @RecipientID IS NULL
		RETURN -1
	
	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @SenderID)
		RETURN -2

	IF NOT EXISTS(SELECT * FROM [dbo].[User] WHERE ID = @RecipientID)
		RETURN -3

	DECLARE @MY_TRANCOUNT INT = @@TRANCOUNT

	IF @MY_TRANCOUNT = 0
		BEGIN TRAN

	BEGIN TRY
		INSERT INTO [dbo].[Message](OwnerID, SenderID, RecipientID, [Message], [State])
		VALUES(@SenderID, @SenderID, @RecipientID, @Message, 'SENT')

		SET @ID = SCOPE_IDENTITY()

		INSERT INTO [dbo].[Message](OwnerID, SenderID, RecipientID, [Message], [State])
		VALUES(@RecipientID, @SenderID, @RecipientID, @Message, 'PENDING')

		IF @MY_TRANCOUNT = 0
			COMMIT TRAN
	END TRY
	BEGIN CATCH
		IF @MY_TRANCOUNT = 0
			ROLLBACK TRAN

		THROW
	END CATCH
END
