﻿/*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
*****************************************************************************/


-- --------------------------------------------
--	SP: Contact_InsUpd
--	Create date: 03/02/2020
--
--	DESCRIPTION
--	Insert / Update a user in the database
--
--	RETURNS
--	-1:		One or more parameter is missing
--	-2:		EMail is already in use
--	-3:		Nickname is already in use
-- --------------------------------------------
CREATE PROCEDURE [dbo].[User_InsUpd]
	@ID			INT				OUTPUT,
	@EMail		NVARCHAR(260),
	@Nickname	NVARCHAR(100),
	@Password	NVARCHAR(100)
AS
BEGIN
	IF ISNULL(@EMail, '') = '' OR ISNULL(@Nickname, '') = '' OR @Password IS NULL
		RETURN -1

	IF EXISTS(SELECT * FROM [dbo].[User] WHERE EMail = @EMail)
		RETURN -2

	IF EXISTS(SELECT * FROM [dbo].[User] WHERE Nickname = @Nickname)
		RETURN -3

	DECLARE @PasswordHash VARBINARY(MAX)
	SET @PasswordHash = HASHBYTES('SHA2_512', @Password)

	IF ISNULL(@ID, 0) = 0
	BEGIN
		INSERT INTO [dbo].[User](EMail, Nickname, [Password]) VALUES(@EMail, @Nickname, @PasswordHash)
		SET @ID = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [dbo].[User] SET EMail		= @EMail, 
								Nickname	= @Nickname, 
								[Password]	= @PasswordHash
		WHERE ID = @ID
	END

	SELECT @ID AS ID
END
