﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//*****************************************************************************

using System;
using System.Threading.Tasks;
using System.Windows.Forms;
using TDP.BaseServices.Infrastructure.DataValidation;
using TDP.ChatClient.Infrastructure.GUI;
using TDP.ChatLib.Messages.Core;
using TDP.ChatLib.Client.Infrastructure;

namespace TDP.ChatClient
{
    public partial class FrmMain : Form
    {
        private BindingSource _BsContacts;
        private BindingSource _BsContactRequests;
        private BindingSource _BsMessages;

        private void LstMessages_DrawItem(object sender, DrawItemEventArgs e)
        {

        }

        private void LstMessages_MeasureItem(object sender, MeasureItemEventArgs e)
        {
            
        }

        private void SetFormStatus(bool enabled, bool resetData = false)
        {
            if (resetData)
            {
                if (_BsContacts != null)
                    _BsContacts.DataSource = null;
                if (_BsContactRequests != null)
                    _BsContactRequests.DataSource = null;
                if (_BsMessages != null)
                    _BsMessages.DataSource = null;
            }

            GrpContacts.Enabled = enabled;
            GrpContactRequests.Enabled = enabled;
            GrpConversation.Enabled = enabled;
        }

        private void SetFormConnected()
        {
            StatusBar1.Items["ConnectionStatus"].Text = "Ready";
            StatusBar1.Items["ConnectionStatus"].Text = "Connected as " + Common.ChatClient.UserNickname + " (" + Common.ChatClient.UserEMail + ")";
            SetFormStatus(true);
        }

        private async Task SynchronizeAccount(bool synchronizeContacts, bool synchronizeContactRequests, bool synchronizeMessages, int recipientID)
        {
            this.MakeUISafeCall(new Action(() => {
                StatusBar1.Items["GeneralStatus"].Text = "Synchronizing account...";
                // Check if we have new messages from the current selected contact, if not we don't update
                // message list but the contact list to get the updated pending messages count
                if (synchronizeMessages && recipientID > 0
                    && LstContacts.SelectedIndex >= 0 && recipientID != (int)LstContacts.SelectedValue)
                {
                    synchronizeMessages = false;
                    synchronizeContacts = true;
                }
            }));
            
            SynchronizeClientResponse SyncRsp = await Common.ChatClient.SynchronizeAsync(synchronizeContacts, synchronizeContactRequests, synchronizeMessages, recipientID);

            if (!SyncRsp.Success)
            {
                this.MakeUISafeCall(new Action(() => {
                    StatusBar1.Items["GeneralStatus"].Text = "An error occurred during client synchronization";
                }));
                return;
            }

            if (synchronizeContacts)
            {
                this.MakeUISafeCall(new Action(() => {
                    _BsContacts.DataSource = SyncRsp.Contacts;
                    LstContacts.ValueMember = "ID";
                    LstContacts.DisplayMember = "ListBoxContactItem";
                    LstContacts.DataSource = _BsContacts;
                }));
            }
                
            if (synchronizeContactRequests)
            {
                this.MakeUISafeCall(new Action(() => {
                    _BsContactRequests.DataSource = SyncRsp.ContactRequestes;
                    LstContactRequests.ValueMember = "ID";
                    LstContactRequests.DisplayMember = "ListBoxContactItem";
                    LstContactRequests.DataSource = _BsContactRequests;
                }));
            }

            if (synchronizeMessages)
            {
                this.MakeUISafeCall(new Action(() => {
                    _BsMessages.DataSource = SyncRsp.Messages;
                    LstMessages.ValueMember = "ID";
                    LstMessages.DisplayMember = "ListBoxMessageItem";
                    LstMessages.DataSource = _BsMessages;
                    //LstMessages.DrawItem += LstMessages_DrawItem;
                    //LstMessages.MeasureItem += LstMessages_MeasureItem;
                }));
            }

            this.MakeUISafeCall(new Action(() => {
                StatusBar1.Items["GeneralStatus"].Text = "Ready";
            }));
        }

        private async Task SendChatMessage(int recipientID, string message)
        {
            this.MakeUISafeCall(new Action(() => {
                StatusBar1.Items["GeneralStatus"].Text = "Sending message...";
            }));

            SendChatMessageResponse SendMessageRsp = await Common.ChatClient.SendMessageAsync(recipientID, message);

            if (!SendMessageRsp.Success)
            {
                this.MakeUISafeCall(new Action(() => {
                    StatusBar1.Items["GeneralStatus"].Text = "An error occurred during message send";
                }));
                return;
            }

            this.MakeUISafeCall(new Action(() => {
               _BsMessages.Add(new MessageItem() { CreationDate = DateTime.Now, SenderNickname = Common.ChatClient.UserNickname, Message = message });
               TxtMessage.Text = string.Empty;
               StatusBar1.Items["GeneralStatus"].Text = "Ready";
            }));
        }


        public FrmMain()
        {
            InitializeComponent();

            StatusBar1.Items["GeneralStatus"].Text = string.Empty;
            StatusBar1.Items["ConnectionStatus"].Text = "Not connected";
            SetFormStatus(false);

            _BsContacts = new BindingSource();
            _BsMessages = new BindingSource();
            _BsContactRequests = new BindingSource();

            if (Common.ChatClient == null)
            {
                Common.ChatClient = new ChatLib.Client.ChatClient();

                FrmLogin FrmLogin = new FrmLogin();
                DialogResult DR = FrmLogin.ShowDialog();

                if (DR == DialogResult.OK)
                    SetFormConnected();
                
                Common.ChatClient.SynchronizeRequested += ChatClient_SynchronizeRequested;
                Common.ChatClient.LoginClientCompleted += ChatClient_LoginClientCompleted;
                Common.ChatClient.RegisterCompleted += ChatClient_RegisterCompleted;
                Common.ChatClient.ConnectionClosed += ChatClient_ConnectionClosed;
            }
        }

        private async void ChatClient_RegisterCompleted(object sender, RegisterClientCompletedEventArgs e)
        {
            SetFormConnected();
            await SynchronizeAccount(true, true, false, 0);
            LstContacts_SelectedIndexChanged(null, null);
        }

        private async void ChatClient_LoginClientCompleted(object sender, LoginClientCompletedEventArgs e)
        {
            SetFormConnected();
            await SynchronizeAccount(true, true, false, 0);
            LstContacts_SelectedIndexChanged(null, null);
        }

        private async void ChatClient_SynchronizeRequested(object sender, SynchronizeRequestedEventArgs e)
        {
            await SynchronizeAccount(e.SynchronizeContacts, e.SynchronizeContactRequests, e.SynchronizeMessages, e.ClientID);
        }

        private void ChatClient_ConnectionClosed(object sender, EventArgs e)
        {
            StatusBar1.Items["ConnectionStatus"].Text = "Not connected";
            SetFormStatus(false, true);
        }

        private async void FrmMain_Load(object sender, EventArgs e)
        {
            if (Common.ChatClient != null)
            {
                if (Common.ChatClient.Connected)
                    await SynchronizeAccount(true, true, false, 0);
            }
        }

        private async void LstContacts_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (LstContacts.SelectedIndex >= 0)
            {
                await SynchronizeAccount(true, false, true, (int)LstContacts.SelectedValue);
            }
        }

        private async void BtnSend_Click(object sender, EventArgs e)
        {
            if (DataValidationHelper.IsEmptyString(TxtMessage.Text))
            {
                WinUtils.ShowAlert("Enter a message");
                return;
            }

            if (LstContacts.SelectedIndex < 0)
            {
                WinUtils.ShowAlert("Select a contact");
                return;
            }

            int RecipientID = (int)LstContacts.SelectedValue;

            await SendChatMessage(RecipientID, TxtMessage.Text);
        }

        private void BtnAddContact_Click(object sender, EventArgs e)
        {
            FrmAddContact FrmAddContact = new FrmAddContact();
            FrmAddContact.ShowDialog();
        }

        private void MnuExit_Click(object sender, EventArgs e)
        {
            if (WinUtils.ShowConfirm("Do you want to exit The Dummy Programmwer Chat?") == DialogResult.Yes)
                Close();
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private async void BtnConfirmRequest_Click(object sender, EventArgs e)
        {
            if(LstContactRequests.SelectedIndex < 0)
            {
                WinUtils.ShowAlert("Select a contact request");
                return;
            }

            if (WinUtils.ShowConfirm("Do you want to confirm the contact request?") == DialogResult.No)
                return;

            GenericResponse R = await Common.ChatClient.ConfirmAddClientRequest((int)LstContactRequests.SelectedValue);

            if (R.Success)
            {
                await SynchronizeAccount(true, true, false, 0);
            }
            else
            {
                WinUtils.ShowAlert("An error occurred while confirming the contact request");
                return;
            }
        }

        private async void BtnRejectRequest_Click(object sender, EventArgs e)
        {
            if (LstContactRequests.SelectedIndex < 0)
            {
                WinUtils.ShowAlert("Select a contact request");
                return;
            }

            if (WinUtils.ShowConfirm("Do you want to reject the contact request?") == DialogResult.No)
                return;

            GenericResponse R = await Common.ChatClient.RejectAddClientRequest((int)LstContactRequests.SelectedValue);

            if (R.Success)
            {
                await SynchronizeAccount(true, true, false, 0);
            }
            else
            {
                WinUtils.ShowAlert("An error occurred while rejecting the contact request");
                return;
            }
        }

        private async void BtnRemoveContact_Click(object sender, EventArgs e)
        {
            if (LstContacts.SelectedIndex < 0)
            {
                WinUtils.ShowAlert("Select a contact");
                return;
            }

            if (WinUtils.ShowConfirm("Do you want to remove the selected contact?") == DialogResult.No)
                return;

            GenericResponse R = await Common.ChatClient.RemoveClient((int)LstContacts.SelectedValue);

            if (R.Success)
            {
                await SynchronizeAccount(true, true, false, 0);
            }
            else
            {
                WinUtils.ShowAlert("An error occurred while removing the contact");
                return;
            }
        }

        private void MnuLogin_Click(object sender, EventArgs e)
        {
            FrmLogin FrmLogin = new FrmLogin();
            DialogResult DR = FrmLogin.ShowDialog();
        }

        private void MnuLogout_Click(object sender, EventArgs e)
        {
            if (WinUtils.ShowConfirm("Do you want to logout?") == DialogResult.Yes)
                Common.ChatClient.Disconnect();
        }

        private void MenuFile_DropDownOpening(object sender, EventArgs e)
        {
            if (Common.ChatClient == null || !Common.ChatClient.Connected)
            {
                MnuLogin.Enabled = true;
                MnuLogout.Enabled = false;
            }
            else if (Common.ChatClient != null && Common.ChatClient.Connected)
            {
                MnuLogin.Enabled = false;
                MnuLogout.Enabled = true;
            }
        }

        private void TxtMessage_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == '\r')
                BtnSend_Click(null, null);
        }
    }
}
