﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//*****************************************************************************

using System;
using System.ComponentModel;
using System.Net;
using System.Net.Sockets;
using System.Threading.Tasks;
using System.Text.RegularExpressions;
using TDP.BaseServices.Infrastructure.Configuration;
using TDP.BaseServices.Infrastructure.Net;
using TDP.BaseServices.Infrastructure.Security;
using TDP.ChatLib.Client.Infrastructure;
using TDP.ChatLib.Client.Infrastructure.Configuration;
using TDP.ChatLib.Messages.Abstract;
using TDP.ChatLib.Messages.Core;
using TDP.PushLib.Client;
using TDP.PushLib.Client.Infrastructure;
using TDP.BaseServices.Infrastructure.Logging.Abstract;
using TDP.BaseServices.Infrastructure.Logging;
using System.IO;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Authentication;
using System.Timers;

namespace TDP.ChatLib.Client
{
    public class ChatClient
    {
        #region "Events definition"
        public event EventHandler RegisterClientStarted;
        protected virtual void OnRegisterClientStarted(EventArgs e)
        {
            EventHandler handler = RegisterClientStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }


        public delegate void RegisterCompletedDelegate(object sender, RegisterClientCompletedEventArgs e);
        public event RegisterCompletedDelegate RegisterCompleted;
        protected virtual void OnRegisterCompleted(RegisterClientCompletedEventArgs e)
        {
            RegisterCompletedDelegate handler = RegisterCompleted;
            if (handler != null)
            {
                foreach (RegisterCompletedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler LoginClientStarted;
        protected virtual void OnLoginClientStarted(EventArgs e)
        {
            EventHandler handler = LoginClientStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void LoginClientCompletedDelegate(object sender, LoginClientCompletedEventArgs e);
        public event LoginClientCompletedDelegate LoginClientCompleted;
        protected virtual void OnLoginClientCompleted(LoginClientCompletedEventArgs e)
        {
            LoginClientCompletedDelegate handler = LoginClientCompleted;
            if (handler != null)
            {
                foreach (LoginClientCompletedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler SynchronizeClientStarted;
        protected virtual void OnSynchronizeClientStarted(EventArgs e)
        {
            EventHandler handler = SynchronizeClientStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void SynchronizeClientCompletedDelegate(object sender, SynchronizeClientCompletedEventArgs e);
        public event SynchronizeClientCompletedDelegate SynchronizeClientCompleted;
        protected virtual void OnSynchronizeClientCompleted(SynchronizeClientCompletedEventArgs e)
        {
            SynchronizeClientCompletedDelegate handler = SynchronizeClientCompleted;
            if (handler != null)
            {
                foreach (SynchronizeClientCompletedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler SendMessageStarted;
        protected virtual void OnSendMessageStarted(EventArgs e)
        {
            EventHandler handler = SendMessageStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void SendMessageCompletedDelegate(object sender, SendMessageCompletedEventArgs e);
        public event SendMessageCompletedDelegate SendMessageCompleted;
        protected virtual void OnSendMessageCompleted(SendMessageCompletedEventArgs e)
        {
            SendMessageCompletedDelegate handler = SendMessageCompleted;
            if (handler != null)
            {
                foreach (SendMessageCompletedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler SearchClientStarted;
        protected virtual void OnSearchClientStarted(EventArgs e)
        {
            EventHandler handler = SearchClientStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void SearchClientCompletedDelegate(object sender, SearchClientCompletedEventArgs e);
        public event SearchClientCompletedDelegate SearchClientCompleted;
        protected virtual void OnSearchClientCompleted(SearchClientCompletedEventArgs e)
        {
            SearchClientCompletedDelegate handler = SearchClientCompleted;
            if (handler != null)
            {
                foreach (SearchClientCompletedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }


        public event EventHandler AddClientStarted;
        protected virtual void OnAddClientStarted(EventArgs e)
        {
            EventHandler handler = AddClientStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void AddClientCompletedDelegate(object sender, AddClientCompletedEventArgs e);
        public event AddClientCompletedDelegate AddClientCompleted;
        protected virtual void OnAddClientCompleted(AddClientCompletedEventArgs e)
        {
            AddClientCompletedDelegate handler = AddClientCompleted;
            if (handler != null)
            {
                foreach (AddClientCompletedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler ConfirmAddClientRequestStarted;
        protected virtual void OnConfirmAddClientRequestStarted(EventArgs e)
        {
            EventHandler handler = ConfirmAddClientRequestStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler ConfirmAddClientRequestCompleted;
        protected virtual void OnConfirmAddClientRequestCompleted(EventArgs e)
        {
            EventHandler handler = ConfirmAddClientRequestCompleted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler RejectAddClientRequestStarted;
        protected virtual void OnRejectAddClientRequestStarted(EventArgs e)
        {
            EventHandler handler = RejectAddClientRequestStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler RejectAddClientRequestCompleted;
        protected virtual void OnRejectAddClientRequestCompleted(EventArgs e)
        {
            EventHandler handler = RejectAddClientRequestCompleted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void SynchronizeRequestedDelegate(object sender, SynchronizeRequestedEventArgs e);
        public event SynchronizeRequestedDelegate SynchronizeRequested;
        protected virtual void OnSynchronizeRequested(SynchronizeRequestedEventArgs e)
        {
            SynchronizeRequestedDelegate handler = SynchronizeRequested;
            if (handler != null)
            {
                foreach (SynchronizeRequestedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public delegate void ConnectionClosedDelegate(object sender, EventArgs e);
        public event ConnectionClosedDelegate ConnectionClosed;
        protected virtual void OnConnectionClosed(EventArgs e)
        {
            ConnectionClosedDelegate handler = ConnectionClosed;
            if (handler != null)
            {
                foreach (ConnectionClosedDelegate singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler RemoveClientStarted;
        protected virtual void OnRemoveClientStarted(EventArgs e)
        {
            EventHandler handler = RemoveClientStarted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        public event EventHandler RemoveClientCompleted;
        protected virtual void OnRemoveClientCompleted(EventArgs e)
        {
            EventHandler handler = RemoveClientCompleted;
            if (handler != null)
            {
                foreach (EventHandler singleCast in handler.GetInvocationList())
                {
                    ISynchronizeInvoke syncInvoke = singleCast.Target as ISynchronizeInvoke;
                    if ((syncInvoke != null) && (syncInvoke.InvokeRequired))
                        syncInvoke.Invoke(singleCast, new object[] { this, e });
                    else
                        singleCast(this, e);
                }
            }
        }

        #endregion
        private static ILogger _Log = new Logger();
        private AuthenticationToken<int> _UserAuthenticationToken;
        private PushClient<int> _PushClient;
        private Timer _Timer;

        public bool Connected { get; private set; }
        public string UserEMail { get; private set; }
        public string UserNickname { get; private set; }

        public ChatClient()
        {
            _Log.Info("ChatClient -> ChatClient() -> started");

            Config.Init();

            _PushClient = new PushClient<int>();
            _PushClient.PushMessageReceived += _PushClient_PushMessageReceived;
            _PushClient.ConnectionClosed += _PushClient_ConnectionClosed;

            _Timer = new Timer();
            _Timer.Interval = Config.AuthTokenCheckExpiringIntervalMS;
            _Timer.Enabled = true;
            _Timer.Elapsed += _Timer_Elapsed;

            _Log.Info("ChatClient -> ChatClient() -> ending");
        }

        private async void _Timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _Timer.Enabled = false;

            if (_UserAuthenticationToken == null)
                return;

            if (_UserAuthenticationToken.IsExpiring(Config.AuthTokenMinutesToRenew))
            {
                Task T = new Task(() =>
                {
                    _Log.Info("ChatClient -> _Timer_Elapsed() -> async task started");

                    RenewTokenResponse RenewRsp = new RenewTokenResponse();
                    
                    try
                    {
                        RenewToken RenewMsg = new RenewToken(_UserAuthenticationToken);
                        RenewRsp = SendMessageToServer<RenewTokenResponse>(RenewMsg);

                        if (RenewRsp.Success)
                            _UserAuthenticationToken = RenewRsp.AuthToken;
                    }
                    catch (Exception ex)
                    {
                        _Log.Error("ChatClient -> _Timer_Elapsed() -> an error occurred", ex);
                    }

                    _Log.Info("ChatClient -> _Timer_Elapsed() -> async task ending");
                });
                T.Start();
                await T;
            }

            _Timer.Enabled = true;
        }

        private void _PushClient_PushMessageReceived(object sender, PushMessageReceivedEventArgs e)
        {
            _Log.Info($"ChatClient -> _PushClient_PushMessageReceived() -> started, message: {e.PushMessage.Message}");

            bool SynchronizeContacts = e.PushMessage.Message.Contains("SynchronizeContacts");
            bool SynchronizeContactRequests = e.PushMessage.Message.Contains("SynchronizeContactRequests");

            Match SynchMatch = Regex.Match(e.PushMessage.Message, @"SynchronizeMessages:(\d{1,})");
            int UserID = 0;
            bool SynchronizeMessages = false;
            if (SynchMatch.Success)
            {
                UserID = int.Parse(SynchMatch.Groups[1].Value);
                SynchronizeMessages = true;
            }

            SynchronizeRequestedEventArgs SynchReqEA = new SynchronizeRequestedEventArgs(SynchronizeContacts, SynchronizeContactRequests, SynchronizeMessages, UserID);
            OnSynchronizeRequested(SynchReqEA);

            _Log.Info("ChatClient -> _PushClient_PushMessageReceived() -> ending");
        }

        private void _PushClient_ConnectionClosed(object sender, EventArgs e)
        {
            _Log.Info("ChatClient -> _PushClient_ConnectionClosed() -> ending");

            OnConnectionClosed(new EventArgs());

            _Log.Info("ChatClient -> _PushClient_ConnectionClosed() -> ending");
        }

        private bool ValidateServerCertificate(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            if (Config.IgnoreSslErrors)
                return true;

            if (sslPolicyErrors == SslPolicyErrors.None)
                return true;

            _Log.Info($"ChatClient -> ValidateServerCertificate() -> certificate error: {sslPolicyErrors}");

            return false;
        }

        private T SendMessageToServer<T>(IMessageBase message)
        {
            Socket SocketClient = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            SocketClient.SendTimeout = Config.SendTimeout;
            SocketClient.ReceiveTimeout = Config.ReceiveTimeout;
            IPAddress MyIPAddress = IPAddress.Parse(Config.ConnectIPAddress);
            IPEndPoint MyEndPoint = new IPEndPoint(MyIPAddress, Config.ConnectPort);
            SocketClient.Connect(MyEndPoint);

            Stream SocketStream;
            SslStream SslSocketStream;
            if (!Config.UseSsl)
            {
                SocketStream = new NetworkStream(SocketClient);
            }
            else
            {
                SslSocketStream = new SslStream(new NetworkStream(SocketClient), false, new RemoteCertificateValidationCallback(ValidateServerCertificate));
                SslSocketStream.AuthenticateAsClient(Config.SslTargetHost);
                SocketStream = SslSocketStream;
            }
            
            SocketHelper.SendObject(SocketStream, message);

            T RegRsp = SocketHelper.ReceiveObject<T>(SocketStream);

            SocketStream.Close();
            SocketStream.Dispose();

            if (SocketClient.Connected)
                SocketClient.Close();
            SocketClient.Dispose();

            return RegRsp;
        }

        public async Task<RegisterClientResponse> RegisterAsync(string email, string nickname, string password)
        {
            Task<RegisterClientResponse> T = new Task<RegisterClientResponse>(() =>
            {
                _Log.Info("ChatClient -> RegisterAsync() -> async task started");

                RegisterClientResponse RegisterRsp = new RegisterClientResponse();

                EventArgs OnRegisterClientEA = new EventArgs();
                OnRegisterClientStarted(OnRegisterClientEA);

                try
                {
                    RegisterClient RegMsg = new RegisterClient(email, nickname, password);
                    RegisterRsp = SendMessageToServer<RegisterClientResponse>(RegMsg);

                    if (RegisterRsp.Success)
                    {
                        _UserAuthenticationToken = RegisterRsp.AuthToken;
                        _PushClient.ConnectAsync(_UserAuthenticationToken);
                        Connected = true;
                        UserEMail = email;
                        UserNickname = nickname;
                    }
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> RegisterAsync() -> an error occurred", ex);
                    RegisterRsp = new RegisterClientResponse(false, int.MinValue, "An error occurred while executing the operation", null);
                }

                RegisterClientCompletedEventArgs OnRegisteredEA = new RegisterClientCompletedEventArgs(RegisterRsp);
                OnRegisterCompleted(OnRegisteredEA);

                _Log.Info("ChatClient -> RegisterAsync() -> async task ending");

                return RegisterRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<LoginClientResponse> LoginAsync(string email, string password)
        {
            Task<LoginClientResponse> T = new Task<LoginClientResponse>(() =>
            {
                _Log.Info("ChatClient -> LoginAsync() -> async task started");

                LoginClientResponse LoginRsp = new LoginClientResponse();
                EventArgs OnLoginClientEA = new EventArgs();
                OnLoginClientStarted(OnLoginClientEA);

                try
                {
                    LoginClient LoginMsg = new LoginClient(email, password);
                    LoginRsp = SendMessageToServer<LoginClientResponse>(LoginMsg);

                    if (LoginRsp.Success)
                    {
                        _UserAuthenticationToken = LoginRsp.AuthToken;
                        _PushClient.ConnectAsync(_UserAuthenticationToken);
                        Connected = true;
                        UserEMail = email;
                        UserNickname = LoginRsp.Nickname;
                    }
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> LoginAsync() -> an error occurred", ex);
                    LoginRsp = new LoginClientResponse(false, int.MinValue, "An error occurred while executing the operation", string.Empty, null);
                }

                LoginClientCompletedEventArgs OnLoginClientCompletedEA = new LoginClientCompletedEventArgs(LoginRsp);
                OnLoginClientCompleted(OnLoginClientCompletedEA);

                _Log.Info("ChatClient -> LoginAsync() -> async task ending");

                return LoginRsp;
            });
            T.Start();
            return await T;
        }

        public void Disconnect()
        {
            Connected = false;
            _PushClient.Disconnect();
        }

        public async Task<SynchronizeClientResponse> SynchronizeAsync(bool synchronizeContacts, bool synchronizeContactRequests, bool synchronizeMessages, int recipientID)
        {
            Task<SynchronizeClientResponse> T = new Task<SynchronizeClientResponse>(() =>
            {
                _Log.Info("ChatClient -> SynchronizeAsync() -> async task started");

                SynchronizeClientResponse SynchronizeRsp = new SynchronizeClientResponse();
                EventArgs OnSynchronizeClientEA = new EventArgs();
                OnSynchronizeClientStarted(OnSynchronizeClientEA);

                try
                {
                    SynchronizeClient SynchronizeMsg = new SynchronizeClient(synchronizeContacts, synchronizeContactRequests, synchronizeMessages, recipientID, _UserAuthenticationToken);
                    SynchronizeRsp = SendMessageToServer<SynchronizeClientResponse>(SynchronizeMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> SynchronizeAsync() -> an error occurred", ex);
                    SynchronizeRsp = new SynchronizeClientResponse(false, int.MinValue, "An error occurred while executing the operation", null, null, null);
                }

                SynchronizeClientCompletedEventArgs OnSynchronizeClientCompletedEA = new SynchronizeClientCompletedEventArgs(SynchronizeRsp);
                OnSynchronizeClientCompleted(OnSynchronizeClientCompletedEA);

                _Log.Info("ChatClient -> SynchronizeAsync() -> async task ending");

                return SynchronizeRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<SendChatMessageResponse> SendMessageAsync(int recipientID, string chatMessage)
        {
            Task<SendChatMessageResponse> T = new Task<SendChatMessageResponse>(() =>
            {
                _Log.Info("ChatClient -> SendMessageAsync() -> async task started");

                SendChatMessageResponse SendMsgRsp = new SendChatMessageResponse();
                EventArgs OnSendMessageEA = new EventArgs();
                OnSendMessageStarted(OnSendMessageEA);

                try
                {
                    SendChatMessage SendMsg = new SendChatMessage(recipientID, chatMessage, _UserAuthenticationToken);
                    SendMsgRsp = SendMessageToServer<SendChatMessageResponse>(SendMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> SendMessageAsync() -> an error occurred", ex);
                    SendMsgRsp = new SendChatMessageResponse(false, int.MinValue, "An error occurred while executing the operation");
                }

                SendMessageCompletedEventArgs OnSendMessageCompletedEA = new SendMessageCompletedEventArgs(SendMsgRsp);
                OnSendMessageCompleted(OnSendMessageCompletedEA);

                _Log.Info("ChatClient -> SendMessageAsync() -> async task ending");

                return SendMsgRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<SearchClientResponse> SearchClientAsync(string searchText)
        {
            Task<SearchClientResponse> T = new Task<SearchClientResponse>(() =>
            {
                _Log.Info("ChatClient -> SearchClientAsync() -> async task started");

                SearchClientResponse SearchClientRsp = new SearchClientResponse();
                EventArgs OnSearchClientEA = new EventArgs();
                OnSearchClientStarted(OnSearchClientEA);

                try
                {
                    SearchClient SearchClientMsg = new SearchClient(searchText, _UserAuthenticationToken);
                    SearchClientRsp = SendMessageToServer<SearchClientResponse>(SearchClientMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> SearchClientAsync() -> an error occurred", ex);
                    SearchClientRsp = new SearchClientResponse(false, int.MinValue, "An error occurred while executing the operation");
                }

                SearchClientCompletedEventArgs OnSearchClientCompletedEA = new SearchClientCompletedEventArgs(SearchClientRsp);
                OnSearchClientCompleted(OnSearchClientCompletedEA);

                _Log.Info("ChatClient -> SearchClientAsync() -> async task ending");

                return SearchClientRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<AddClientResponse> AddClientAsync(int id)
        {
            Task<AddClientResponse> T = new Task<AddClientResponse>(() =>
            {
                _Log.Info("ChatClient -> AddClientAsync() -> async task started");

                AddClientResponse AddClientRsp = new AddClientResponse();
                EventArgs OnAddClientEA = new EventArgs();
                OnAddClientStarted(OnAddClientEA);

                try
                {
                    AddClient AddClientMsg = new AddClient(id, _UserAuthenticationToken);
                    AddClientRsp = SendMessageToServer<AddClientResponse>(AddClientMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> AddClientAsync() -> an error occurred", ex);
                    AddClientRsp = new AddClientResponse(false, int.MinValue, "An error occurred while executing the operation");
                }

                AddClientCompletedEventArgs OnAddClientCompletedEA = new AddClientCompletedEventArgs(AddClientRsp);
                OnAddClientCompleted(OnAddClientCompletedEA);

                _Log.Info("ChatClient -> AddClientAsync() -> async task ending");

                return AddClientRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<GenericResponse> ConfirmAddClientRequest(int id)
        {
            Task<GenericResponse> T = new Task<GenericResponse>(() =>
            {
                _Log.Info("ChatClient -> ConfirmAddClientRequest() -> async task started");

                GenericResponse ConfirmAddClientReqRsp = new GenericResponse();
                EventArgs OnConfirmAddClientReqEA = new EventArgs();
                OnConfirmAddClientRequestStarted(OnConfirmAddClientReqEA);

                try
                {
                    ConfirmAddClientRequest ConfirmAddClientReqMsg = new ConfirmAddClientRequest(id, _UserAuthenticationToken);
                    ConfirmAddClientReqRsp = SendMessageToServer<GenericResponse>(ConfirmAddClientReqMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> ConfirmAddClientRequest() -> an error occurred", ex);
                    ConfirmAddClientReqRsp = new GenericResponse(false, int.MinValue, "An error occurred while executing the operation");
                }

                EventArgs OnConfirmAddClientReqCompletedEA = new EventArgs();
                OnConfirmAddClientRequestCompleted(OnConfirmAddClientReqCompletedEA);

                _Log.Info("ChatClient -> ConfirmAddClientRequest() -> async task ending");

                return ConfirmAddClientReqRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<GenericResponse> RejectAddClientRequest(int id)
        {
            Task<GenericResponse> T = new Task<GenericResponse>(() =>
            {
                _Log.Info("ChatClient -> RejectAddClientRequest() -> async task started");

                GenericResponse RejectAddClientReqRsp = new GenericResponse();
                EventArgs OnRejectAddClientReqEA = new EventArgs();
                OnRejectAddClientRequestStarted(OnRejectAddClientReqEA);

                try
                {
                    RejectAddClientRequest RejectAddClientReqMsg = new RejectAddClientRequest(id, _UserAuthenticationToken);
                    RejectAddClientReqRsp = SendMessageToServer<GenericResponse>(RejectAddClientReqMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> RejectAddClientRequest() -> an error occurred", ex);
                    RejectAddClientReqRsp = new GenericResponse(false, int.MinValue, "An error occurred while executing the operation");
                }

                EventArgs OnRejectAddClientReqCompletedEA = new EventArgs();
                OnRejectAddClientRequestCompleted(OnRejectAddClientReqCompletedEA);

                _Log.Info("ChatClient -> RejectAddClientRequest() -> async task ending");

                return RejectAddClientReqRsp;
            });
            T.Start();
            return await T;
        }

        public async Task<GenericResponse> RemoveClient(int id)
        {
            Task<GenericResponse> T = new Task<GenericResponse>(() =>
            {
                _Log.Info("ChatClient -> RemoveClient() -> async task started");

                GenericResponse RemoveClientRsp = new GenericResponse();
                EventArgs OnRemoveClientEA = new EventArgs();
                OnRemoveClientStarted(OnRemoveClientEA);

                try
                {
                    RemoveClient RemoveClientMsg = new RemoveClient(id, _UserAuthenticationToken);
                    RemoveClientRsp = SendMessageToServer<GenericResponse>(RemoveClientMsg);
                }
                catch (Exception ex)
                {
                    _Log.Error("ChatClient -> RemoveClient() -> an error occurred", ex);
                    RemoveClientRsp = new GenericResponse(false, int.MinValue, "An error occurred while executing the operation");
                }

                EventArgs OnRemoveClientCompletedEA = new EventArgs();
                OnRemoveClientCompleted(OnRemoveClientCompletedEA);
                
                _Log.Info("ChatClient -> RemoveClient() -> async task ending");

                return RemoveClientRsp;
            });
            T.Start();
            return await T;
        }
    }
}
