﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//*****************************************************************************

using System.Collections.Generic;
using TDP.BaseServices.Infrastructure;
using TDP.BaseServices.Infrastructure.DataAccess;
using TDP.BaseServices.Infrastructure.DataAccess.Abstract;
using TDP.ChatLib.Server.Infrastructure.ChatRepository.Abstract;
using TDP.ChatLib.Server.Models;

namespace TDP.ChatLib.Server.Infrastructure.ChatRepository
{
    class ChatRepositoryManager : IChatRepositoryManager
    {
        private IDBAccess _DbAccess;

        public ChatRepositoryManager(IDBAccess dbAccess)
        {
            _DbAccess = dbAccess;
        }

        public MethodResult<RegisterModel> Register(string email, string nickname, string password)
        {
            DBResult<RegisterModel> DBResult = _DbAccess.StoredProcedure("User_InsUpd")
                                                                .Parameter("@ID", 0, true)
                                                                .Parameter("@EMail", email)
                                                                .Parameter("@Nickname", nickname)
                                                                .Parameter("@Password", password)
                                                                .ExecuteT<RegisterModel>();

            MethodResult<RegisterModel> Result = new MethodResult<RegisterModel>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.TableFirstRow();

            return Result;
        }

        public MethodResult<LoginModel> Login(string email, string password)
        {
            DBResult<LoginModel> DBResult = _DbAccess.StoredProcedure("User_Login")
                                                                .Parameter("@EMail", email)
                                                                .Parameter("@Password", password)
                                                                .ExecuteT<LoginModel>();

            MethodResult<LoginModel> Result = new MethodResult<LoginModel>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.TableFirstRow();

            return Result;
        }

        public MethodResult<List<ContactModel>> GetUserContacts(int userID)
        {
            DBResult<ContactModel> DBResult = _DbAccess.StoredProcedure("User_GetContacts")
                                                                .Parameter("@UserID", userID)
                                                                .ExecuteT<ContactModel>();

            MethodResult<List<ContactModel>> Result = new MethodResult<List<ContactModel>>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.Table;

            return Result;
        }

        public MethodResult<List<ContactRequestModel>> GetUserContactRequests(int userID)
        {
            DBResult<ContactRequestModel> DBResult = _DbAccess.StoredProcedure("User_GetContactRequests")
                                                                .Parameter("@UserID", userID)
                                                                .ExecuteT<ContactRequestModel>();

            MethodResult<List<ContactRequestModel>> Result = new MethodResult<List<ContactRequestModel>>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.Table;

            return Result;
        }

        public MethodResult<List<MessageModel>> GetUserMessages(int userID, int recipientID)
        {
            DBResult<MessageModel> DBResult = _DbAccess.StoredProcedure("User_GetMessages")
                                                                .Parameter("@UserID", userID)
                                                                .Parameter("@RecipientID", recipientID)
                                                                .ExecuteT<MessageModel>();

            MethodResult<List<MessageModel>> Result = new MethodResult<List<MessageModel>>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.Table;

            return Result;
        }

        public int SendMessage(int senderID, int recipientID, string message)
        {
            DBResult DBResult = _DbAccess.StoredProcedure("Message_Send")
                                                                .Parameter("@ID", 0, true)
                                                                .Parameter("@SenderID", senderID)
                                                                .Parameter("@RecipientID", recipientID)
                                                                .Parameter("@Message", message)
                                                                .Execute();

            return DBResult.ReturnValue;
        }

        public MethodResult<List<UserModel>> SearchClient(int userID, string searchText)
        {
            DBResult<UserModel> DBResult = _DbAccess.StoredProcedure("User_Search")
                                                                .Parameter("@UserID", userID)
                                                                .Parameter("@SearchText", searchText)
                                                                .ExecuteT<UserModel>();

            MethodResult<List<UserModel>> Result = new MethodResult<List<UserModel>>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.Table;

            return Result;
        }

        public int AddClient(int requestorID, int recipientID)
        {
            DBResult DBResult = _DbAccess.StoredProcedure("ContactRequest_Send")
                                                                .Parameter("@ID", 0, true)
                                                                .Parameter("@RequestorID", requestorID)
                                                                .Parameter("@RecipientID", recipientID)
                                                                .Execute();

            return DBResult.ReturnValue;
        }

        public int ConfirmAddClientRequest(int userID, int requestID)
        {
            DBResult DBResult = _DbAccess.StoredProcedure("ContactRequest_Confirm")
                                                                .Parameter("@UserID", userID)
                                                                .Parameter("@RequestID", requestID)
                                                                .Execute();

            return DBResult.ReturnValue;
        }

        public int RejectAddClientRequest(int userID, int requestID)
        {
            DBResult DBResult = _DbAccess.StoredProcedure("ContactRequest_Reject")
                                                                .Parameter("@UserID", userID)
                                                                .Parameter("@RequestID", requestID)
                                                                .Execute();

            return DBResult.ReturnValue;
        }

        public MethodResult<List<ContactRequestDetailModel>> GetContactRequestDetail(int ID)
        {
            DBResult<ContactRequestDetailModel> DBResult = _DbAccess.StoredProcedure("ContactRequest_Detail")
                                                                .Parameter("@ID", ID)
                                                                .ExecuteT<ContactRequestDetailModel>();

            MethodResult<List<ContactRequestDetailModel>> Result = new MethodResult<List<ContactRequestDetailModel>>();
            Result.ResultCode = DBResult.ReturnValue;
            Result.ObjectResult = DBResult.Table;

            return Result;
        }

        public int RemoveClient(int userID, int userContactID)
        {
            DBResult DBResult = _DbAccess.StoredProcedure("Contact_Remove")
                                                                .Parameter("@UserID", userID)
                                                                .Parameter("@UserContactID", userContactID)
                                                                .Execute();

            return DBResult.ReturnValue;
        }
    }
}
